from abaqus import session
from abaqus.Odb.OdbCommands import *  # noqa # pylint: disable=unused-import
from abaqus.Property.PlyStackPlot import (  # noqa # pylint: disable=unused-import
    OdbPlyStackPlot,
)
from abaqusConstants import *  # noqa # pylint: disable=unused-import

session.Viewport(name="Viewport: 1")
